
<!-- index.xsl Echelon Resource Report Generator index xsl transformation stylesheet -->
<!-- Echelon Resource Report Generator 4.01.07 -->
<!-- Copyright (C) 2001 - 2009 Echelon Corporation -->
<!-- All Rights reserved -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

    <xsl:import href="rrg.xsl"/>
    <xsl:output method="html" indent="yes" media-type="text/html" encoding="iso-8859-1"/>

    <!-- +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++    -->
    <!-- root-level template. This is where the ball starts to roll.                                            -->
    <!-- +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++    -->
    <xsl:template match="/">

        <html> 
            <!-- get the standard header and body framework created. The Generate-Body template in -->
            <!-- turn invokes "Generate-Core", which in turn provides content to the inner HTML table -->
            <xsl:call-template name="Generate-Head">
                <xsl:with-param name="title" select="$group"/>
            </xsl:call-template>
            <xsl:call-template name="Generate-Body">
                <xsl:with-param name="title" select="$group"/>
            </xsl:call-template>

        </html>
    </xsl:template>

    <!-- +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++    -->
    <!-- Generate-Toolbar() produces the toolbar in the top right corner. The template is being called from     -->
    <!-- Generate-Core.                                                                                         -->
    <!-- +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++    -->
    <xsl:template name="Generate-Toolbar">
        <xsl:comment> template name="Generate-Toolbar" </xsl:comment>
        <a>
            <xsl:attribute name="href">
                <xsl:value-of select="$root-index"/>
            </xsl:attribute>
            Root</a>
        <xsl:value-of select="$nbsp"/>

        <a href="javascript:Transform('{$relative-self}', '{$item-transformation}', '/ldrf/items')">
            Show All</a>
        <xsl:value-of select="$nbsp"/>

        <!-- the DRF guidelines do not request the "fixed URL" string, which is part of every device        -->
        <!-- resource file's header, to follow a particular format. Customers may set it to be an email     -->
        <!-- address, or a URL to a homepage, or anything else. Email and homepage URL are the most common  -->
        <!-- cases, and we choose to automatically support those with the matching hyperling syntax.        -->
        <a>
            <xsl:variable name="fixed-url" select="/ldrf/control/url"/>
            <xsl:choose>
                <xsl:when test="contains($fixed-url, '@')">
                    <xsl:attribute name="href">
                        mailto:<xsl:value-of select="$fixed-url"/>
                    </xsl:attribute>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:attribute name="href">
                        <xsl:value-of select="$fixed-url"/>
                    </xsl:attribute>
                </xsl:otherwise>                
            </xsl:choose>
            Contact</a>
        <xsl:value-of select="$nbsp"/>
        <a href="http://www.lonmark.org">
            LonMark</a>             
    </xsl:template>

    <!-- +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++    -->
    <!-- Generate-Core() is the core of the transformation. The root template calls Generate-Body, which in     -->
    <!-- turn calls Generate-Core to produce the main content of the output (the inner table)                   -->
    <!-- +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++    -->
    <xsl:template name="Generate-Core">
        <xsl:comment> template name="Generate-Core" </xsl:comment>
        <!-- details table -->
        <table width="100%" border="0" cellspacing="0" cellpadding="3" bgcolor="#F0F7FF">

            <!-- top core row: information about the index page, the set it applies to, etc:    -->
            <tr> 
                <td width="100%" colspan="3" align="left" valign="middle">
                    <p>
                        <xsl:comment> some generic information </xsl:comment>
                        <xsl:value-of select="$group"/><xsl:value-of select="$nbsp"/><xsl:value-of select="$set"/>.<xsl:value-of select="$spacer"/>
                        <xsl:comment> resource description string </xsl:comment>
                        <xsl:value-of select="key('strings', $description-string)"/><xsl:value-of select="$spacer"/>
                        <xsl:comment> resource creator string </xsl:comment>
                        <xsl:value-of select="key('strings', $creator-string)"/><xsl:value-of select="$spacer"/>
                        <br/>
                        <xsl:comment> content imported from usage.xml </xsl:comment>
                        <!-- insert user-defined (optional) usage hints: -->
                        <!-- note we allow for both the set-specific hints to be tagged with the word "set", -->
                        <!-- or to be tagged with the full set ID (e.g. "standard 00:00:00:00:00:00:00:00-0" -->
                        <xsl:call-template name="insert-usage-hint">
                            <xsl:with-param name="id" select="'set'"/>
                        </xsl:call-template>
                        <xsl:call-template name="insert-usage-hint">
                            <xsl:with-param name="id" select="$set"/>
                        </xsl:call-template>
                    </p>
                </td>
            </tr>

            <tr>
                <td width="100%" colspan="3" align="left">
                    <font size="-1">
                        Click here for <a href="#VersionInfo">version information.</a>
                    </font>
                </td>
            </tr>

            <!-- quickfind row: textbox and "find!" button -->
            <tr>
                <td width="100%" colspan="3" align="right">
                    <form name="ResourceLocator">
                        <input type="text" name="ResourceName"/>
                        <input type="button" value=" Go! " onclick="javascript:FindResource();"/>
                    </form>
                </td>
            </tr>
            <tr>
                <td width="100%" colspan="3" align="right">
                    <font size="-1">
                        Type the name of a resource listed here, a fragment of such name,
                        <br/>or a regular expression that describes such a resource name,
                        <br/>and hit Go!, or browse the resources by name or index below.
                    </font>
                </td>
            </tr>

            <!-- index header row, sorted by name and by index -->
            <tr align="left" bordercolor="gray" border="1" >
                <th colspan="1" bgcolor="gray">By Name</th>
                <th colspan="2" bgcolor="gray">By Index</th>
            </tr>
            <!-- index data row, sorted by name and by index -->
            <tr>
                <td width="50%">
                    <xsl:for-each select="$items/cpt | $items/nvt | $items/fpt | $items/enumt">
                        <xsl:sort select="@id"/>

                        <a href="javascript:Transform('{$relative-self}', '{$item-transformation}', '/ldrf/items/*[@id=&quot;{@id}&quot;]')">
                            <xsl:value-of select="@id"/>
                        </a>
                        <br/>
                    </xsl:for-each>
                </td>
                <td width="10%">
                    <font color="gray">
                    <xsl:for-each select="$items/cpt | $items/nvt | $items/fpt | $items/enumt">
                        <xsl:sort select="@idx" data-type="number"/>
                            <a href="javascript:Transform('{$relative-self}', '{$item-transformation}', '/ldrf/items/*[@id=&quot;{@id}&quot;]')">
                                <xsl:value-of select="@idx"/>
                            </a>
                            <br/>
                        </xsl:for-each>
                    </font>
                </td>                   
                <td width="40%">
                    <xsl:for-each select="$items/cpt | $items/nvt | $items/fpt | $items/enumt">
                        <xsl:sort select="@idx" data-type="number"/>
                        <a href="javascript:Transform('{$relative-self}', '{$item-transformation}', '/ldrf/items/*[@id=&quot;{@id}&quot;]')">
                            <xsl:value-of select="@id"/>
                        </a>
                        <br/>
                    </xsl:for-each>
                </td>                   
            </tr>

            <tr>
                <td width="100%" colspan="3" align="left" valign="middle">
                    <xsl:value-of select="$spacer"/>
                </td>
            </tr>
            <tr>
                <td width="100%" colspan="3" align="left" valign="middle">
                    <a name="VersionInfo">
                        <h3>
                            Resource File Set Version Information
                        </h3>
                    </a>
                    <div align="left">
                        <p>
                            Data Version: <xsl:value-of select="$spacer"/> <xsl:value-of select="/ldrf/control/data-version"/>
                            <br/>
                            Format Version: <xsl:value-of select="$spacer"/> <xsl:value-of select="/ldrf/control/format-version"/>
                            <br/>
                            Timestamp: <xsl:value-of select="$spacer"/> <xsl:value-of select="/ldrf/control/timestamp"/>
                        </p>
                    </div>
                    <div align="right">
                        <a href="#top">Top of Page</a>
                    </div>
                </td>
            </tr>

        </table>
    </xsl:template>

    <!-- +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++    -->
    <!-- The following four templates are dummy templates that must be implemented although they are not being  -->
    <!-- used in the context of this transformation. The general utilities in rrg.xsl contain conditional calls -->
    <!-- to the following templates, and the XSLT engine must be able to resolve all references to templates in -->
    <!-- all cases. Hence, we declare these templates and make them all show the same error message.            -->
    <!-- +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++    -->
    <xsl:template name="Generate-Nvt-Core">
        <xsl:comment> template name="Generate-Nvt-Core" </xsl:comment>
        <xsl:call-template name="wrong-core-generator-error"/>
    </xsl:template>

    <xsl:template name="Generate-Cpt-Core">
        <xsl:comment> template name="Generate-Cpt-Core" </xsl:comment>
        <xsl:call-template name="wrong-core-generator-error"/>
    </xsl:template>

    <xsl:template name="Generate-Fpt-Core">
        <xsl:comment> template name="Generate-Fpt-Core" </xsl:comment>
        <xsl:call-template name="wrong-core-generator-error"/>
    </xsl:template>

    <xsl:template name="Generate-Enumt-Core">
        <xsl:comment> template name="Generate-Enumt-Core" </xsl:comment>
        <xsl:call-template name="wrong-core-generator-error"/>
    </xsl:template>

</xsl:stylesheet>
