
<!-- rrg.xsl Echelon Resource Report Generator core xsl transformation stylesheet -->
<!-- Echelon Resource Report Generator 4.01.07 -->
<!-- Copyright (C) 2001 - 2009 Echelon Corporation -->
<!-- All Rights reserved -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

    <!-- some global variables: -->

    <!-- +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++    -->
    <!-- note that most of these variables appear in generated <src:img> and <a:href> attributes, which both    -->
    <!-- are subject to special encoding rules if method="html" is specified in the xsl:output element (white-  -->
    <!-- appears as &xZZ; codes in the generated output for those targets, including end-of-line codes). The    -->
    <!-- consequence is that one must be extremely careful with whitespace when defining the values of those    -->
    <!-- variables; the following variable declarations may therefore appear to be ill-formatted, as the        -->
    <!-- outermost closing tag ("/xsl:variable") terminates the value declaration line.                         -->
    <!-- For those variable values that are concatenated from different sources, the alternative approach of    -->
    <!-- using the 'concat()' function (e.g. <xsl:variable name="script" select="concat('/ldrf/control/tools',  -->
    <!-- '/rrg.js')"/>) might seem obvious. However, the process that is used to create the this stylesheet     -->
    <!-- using markup-definition shared with the Resource Report Generator Wizard utilty sadly prevents this    -->
    <!-- alternative from being used.                                                                           -->
    <!-- +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++    -->

    <!-- full path and name of the javascript file (required when creating the HTML header -->
    <xsl:variable name="script">
        <xsl:value-of select="/ldrf/control/tools"/>/rrg.js</xsl:variable>

    <!-- full path and name of the index transformation stylesheet -->
    <xsl:variable name="index-transformation" select="/ldrf/control/tools/xsl"/>

    <!-- full path and name of item transformation stylesheet -->
    <xsl:variable name="item-transformation">
        <xsl:value-of select="/ldrf/control/tools"/>/types.xsl</xsl:variable>

    <!-- name of the XML file currently under inspection -->
    <xsl:variable name="self" select="/ldrf/control/self"/>

    <!-- base location of this XML -->
    <xsl:variable name="base" select="/ldrf/control/base"/>

    <!-- folder containing the XML file currently under inspection -->
    <xsl:variable name="folder" select="/ldrf/control/folder"/>

    <!-- group: what kind of types do we find in here? -->
    <xsl:variable name="group" select="/ldrf/control/group"/>

    <!-- folder and name of self -->
    <xsl:variable name="relative-self">
        <xsl:value-of select="$folder"/>/<xsl:value-of select="$self"/></xsl:variable>

    <!-- name of the resource set currently under inspection -->
    <xsl:variable name="set" select="/ldrf/control/set"/>

    <!-- current selector (this is the resource file scope, not a LonTalk NV selector!) -->
    <xsl:variable name="selector" select="/ldrf/control/selector"/>

    <!-- full name and version number of the tool that generated the XML data -->
    <xsl:variable name="generator" select="/ldrf/control/generator"/>

    <!-- full URL to the 'art' folder (subfolder to the tools area) -->
    <xsl:variable name="art">
        <xsl:value-of select="/ldrf/control/tools"/>/art</xsl:variable>

    <!-- full URL to the 'art' folder (subfolder to the tools area) -->
    <xsl:variable name="lonmark-logo">
        <!-- full URL to the 'art' folder (subfolder to the tools area) -->
        <xsl:value-of select="$art"/>/lonmark.gif</xsl:variable>

    <xsl:variable name="nbsp">
        <!-- writes a HTML &nbsp; entity into the output stream -->
        <xsl:text disable-output-escaping="yes">&#160;</xsl:text>
    </xsl:variable>

    <xsl:variable name="spacer">
        <!-- writes a HTML &nbsp; entity, followed by a normal space, into the output stream -->
        <xsl:text disable-output-escaping="yes">&#160; </xsl:text>
    </xsl:variable>

    <!-- name of the root index (html) page -->
    <xsl:variable name="root-index" select="/ldrf/control/html"/>

    <!-- all items reported in this xml -->
    <xsl:variable name="items" select="/ldrf/items"/>

    <!-- a key for quick access to all string resources -->
    <xsl:key name="strings" match="strings/string" use="@id"/>

    <!-- string-id of the "description" string for the set currently under inspection -->
    <xsl:variable name="description-string" select="/ldrf/control/description/@id"/>

    <!-- string-id of the "creator" string for the set currently under inspection -->
    <xsl:variable name="creator-string" select="/ldrf/control/creator/@id"/>

    <!-- +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++    -->
    <!-- Generate-Head generates the HTML header. Be careful when inserting 'meta' tags, as those are subject   -->
    <!-- to special processing  by the XSLT engine; see standard documentation for xsl:output for more details. -->
    <!-- Generator-Head is being called by the currently active root XSLT stylesheet (the one that imported     -->
    <!-- this one), with the title parameter being some informative but short content description. The set name -->
    <!-- and its program ID will automatically and always be appended by Generate-Head.                         -->
    <!-- +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++    -->

    <xsl:template name="Generate-Head">
        <xsl:param name="title"/>
        <xsl:comment>template name="Generate-Head"</xsl:comment>

        <head>

            <meta name="GENERATOR" content="Echelon Resource Report Generator"/>
            <title> Device Resource Report <xsl:value-of select="$title"/> <xsl:value-of select="$nbsp"/>-<xsl:value-of select="$nbsp"/> <xsl:value-of select="$set"/> </title>

            <script language="JavaScript" src= "{$script}" > </script>

        </head>
    </xsl:template>

    <!-- +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++    -->
    <!-- Generate-Body generates the HTML body. It is being called with the same descriptive title parameter as -->
    <!-- generate-Head. Generate-Body produces the framework and calls "Generate-Core" (without parameters).    -->
    <!-- Generate-Core, in turn, produces the actual content of the page.                                       -->
    <!-- The optional parameter "caller" can be used to select different core generators as detailed here:      -->
    <!-- caller="nvt" generator=Generate-Nvt-Core                                                               -->
    <!-- caller="cpt" generator=Generate-Cpt-Core                                                               -->
    <!-- caller="fpt" generator=Generate-Fpt-Core                                                               -->
    <!-- caller="enumt" generator=Generate-Enumt-Core                                                           -->
    <!-- The template also calls "generator-toolbar", which must be provided by the caller and must produce     -->
    <!-- a (possibly empty) toolbar with icons and links.                                                       -->
    <!-- +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++    -->
    <xsl:template name="Generate-Body">
        <xsl:param name="title"/>
        <xsl:param name="caller" select="default"/>
        <xsl:comment>template name="Generate-Body"</xsl:comment>

        <body onload="Init()">
            <a name="top"/>
            <table width="100%" border="0" cellspacing="0" cellpadding="3" bgcolor="#F0F7FF">
                <tr valign="middle">
                    <td bgcolor="#0089A6" width="10%">
                        <xsl:value-of select="$nbsp"/>
                    </td>
                    <td bgcolor="#FE9B17" width="80%">
                        <h1> 
                            <a>
                                <xsl:attribute name="name"><xsl:value-of select="$title"/></xsl:attribute>
                                <xsl:value-of select="$title"/>
                            </a> 
                        </h1>
                    </td>
                    <td bgcolor="#FE9B17" width="10%" align="right">
                        <a href="http://www.lonmark.org"><img src= "{$lonmark-logo}" border="0" alt="LonMark Association"/></a> 
                    </td>
                </tr>
                <tr>
                    <td bgcolor="#E42330" width="10%">
                        <xsl:value-of select="$nbsp"/>
                    </td>
                    <td width="90%" colspan="2" align="right">
                        <xsl:call-template name="Generate-Toolbar"/>
                    </td>
                </tr>
                <tr>
                    <td bgcolor="#E42330" width="10%">
                        <xsl:value-of select="$nbsp"/>
                    </td>
                    <td width="90%" colspan="2">
                        <xsl:choose>
                            <xsl:when test="$caller = &apos;nvt&apos;">
                                <xsl:call-template name="Generate-Nvt-Core"/>
                            </xsl:when>
                            <xsl:when test="$caller = &apos;cpt&apos;">
                                <xsl:call-template name="Generate-Cpt-Core"/>
                            </xsl:when>
                            <xsl:when test="$caller = &apos;fpt&apos;">
                                <xsl:call-template name="Generate-Fpt-Core"/>
                            </xsl:when>
                            <xsl:when test="$caller = &apos;enumt&apos;">
                                <xsl:call-template name="Generate-Enumt-Core"/>
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:call-template name="Generate-Core"/>
                            </xsl:otherwise>
                        </xsl:choose>
                    </td>
                </tr>
                <tr>
                    <td bgcolor="#E42330" width="10%">
                        <xsl:value-of select="$nbsp"/>
                    </td>
                    <td width="90%" colspan="2" align="right" valign="bottom">
                        <p><font size="-2"> <xsl:value-of select="$generator"/> <br/>Copyright (C) 2001 - 2009 <a href="http://www.echelon.com">Echelon Corporation</a> </font></p>
                    </td>
                </tr>
            </table>

        </body> 
    </xsl:template>

    <xsl:template name="wrong-core-generator-error">
        <xsl:comment> template name="wrong-core-generator-error" </xsl:comment>
        <h1>
            Transformation Error:
        </h1>
        <p>
            The stylesheets you are using provoked a transformation error: <br/>
            An unexpected core generator template was being invoked.<br/>
            Make sure to use the latest version of the Echelon Resource Report Generator and of the 
            XSLT transformation stylesheets and utilities in <xsl:value-of select="/ldrf/control/tools"/>.<br/>
            Contact the <a href="http://www.lonmark.org">LonMark Association</a> or <a href="http://www.echelon.com">Echelon Corporation</a> 
            for possibly available updates.
        </p>
    </xsl:template>

    <!-- +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++    -->
    <!-- following are a couple of useful utility templates                                                     -->
    <!-- +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++    -->

    <xsl:template match="formats">
        <xsl:comment> template match="formats" </xsl:comment>
        <xsl:for-each select="format">
            <xsl:sort select="@id"/>
            <p>
                <xsl:value-of select="@id"/>:<xsl:value-of select="$spacer"/><xsl:value-of select="."/>
            </p>
        </xsl:for-each>
    </xsl:template>

    <xsl:template match="refs">
        <xsl:comment> template match="refs" </xsl:comment>
        <p>
            <xsl:for-each select="ref">
                <xsl:sort select="."/>
                <a href="javascript:Transform('{@href}', '{$item-transformation}', '/ldrf/items/*[@id=&quot;{.}&quot;]')">
                    <xsl:value-of select="."/></a>
                <xsl:value-of select="$spacer"/>
            </xsl:for-each>
        </p>
    </xsl:template>

    <xsl:template name="insert-usage-hint">
        <xsl:param name="id"/>                                      <!-- the id of the usage hint to fetch -->
        <xsl:param name="usage-xml" select="'usage.xml'"/>          <!-- optional: base name of usage hint file -->
        <xsl:param name="usage-path" select="$folder"/>             <!-- optional: location of usage hint hile, relative to root index HTML -->
        <xsl:comment> template name="insert-usage-hint" </xsl:comment>

        <xsl:variable name="usage-file">
            <xsl:value-of select="concat(string($base), string('/'), string($usage-path), string('/'), string($usage-xml))"/>
        </xsl:variable>

        <xsl:copy-of select="document($usage-file)/usage/hints/hint[@id=$id]"/>

    </xsl:template>

</xsl:stylesheet>

