
<!-- types.xsl Echelon Resource Report Generator type xsl transformation stylesheet -->
<!-- Echelon Resource Report Generator 4.01.07 -->
<!-- Copyright (C) 2001 - 2009 Echelon Corporation -->
<!-- All Rights reserved -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

    <xsl:import href="rrg.xsl"/>
    <xsl:output method="html" indent="yes" media-type="text/html" encoding="iso-8859-1"/>

    <!-- +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++    -->
    <!-- root-level templates, handling the base resource (nvt, cpt, fpt, and enumeration (enumt)).             -->
    <!-- This is where the ball starts to roll. Note the core generators are selected, according to the         -->
    <!-- resource being reported                                                                                -->
    <!-- +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++    -->
    <xsl:template match="nvt | cpt | fpt | enumt">
        <html>
            <!-- get the standard header and body framework created. The Generate-Body template in -->
            <!-- turn invokes "Generate-Core", which in turn provides content to the inner HTML table -->
            <xsl:call-template name="Generate-Head">
                <xsl:with-param name="title" select="@id"/>
            </xsl:call-template>
            <xsl:call-template name="Generate-Body">
                <xsl:with-param name="title" select="@id"/>
                <xsl:with-param name="caller" select="name()"/>
            </xsl:call-template>
        </html>
    </xsl:template>

    <!-- +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++    -->
    <!-- Generate-Toolbar() produces the toolbar in the top right corner. The template is being called from     -->
    <!-- Generate-Core.                                                                                         -->
    <!-- +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++    -->
    <xsl:template name="Generate-Toolbar">
        <xsl:comment> template name="Generate-Toolbar" </xsl:comment>
        <a>
            <xsl:attribute name="href">
                <xsl:value-of select="$root-index"/>
            </xsl:attribute>Root</a>
        <xsl:value-of select="$nbsp"/>

        <a href="javascript:print()">Print</a>
        <xsl:value-of select="$nbsp"/>

        <!-- the DRF guidelines do not request the "fixed URL" string, which is part of every device        -->
        <!-- resource file's header, to follow a particular format. Customers may set it to be an email     -->
        <!-- address, or a URL to a homepage, or anything else. Email and homepage URL are the most common  -->
        <!-- cases, and we choose to automatically support those with the matching hyperling syntax.        -->
        <a>
            <xsl:variable name="fixed-url" select="/ldrf/control/url"/>
            <xsl:choose>
                <xsl:when test="contains($fixed-url, '@')">
                    <xsl:attribute name="href">
                        mailto:<xsl:value-of select="$fixed-url"/>
                    </xsl:attribute>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:attribute name="href">
                        <xsl:value-of select="$fixed-url"/>
                    </xsl:attribute>
                </xsl:otherwise>                
            </xsl:choose>
            Contact</a>
        <xsl:value-of select="$nbsp"/>
        <a href="http://www.lonmark.org">
            LonMark</a>             
    </xsl:template>

    <!-- +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++    -->
    <!-- Generate-Core() is the core of the transformation. The root template calls Generate-Body, which in     -->
    <!-- turn calls Generate-Core to produce the main content of the output (the inner table)                   -->
    <!-- +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++    -->
    <xsl:template name="Generate-Nvt-Core">
        <xsl:comment> template name="Generate-Nvt-Core" </xsl:comment>

        <!-- details table -->
        <table width="100%" border="0" cellspacing="0" cellpadding="3" bgcolor="#F0F7FF">

        <xsl:call-template name="verbose-resource-description"/>
        <xsl:call-template name="report-standard-aspects"/>

        <xsl:call-template name="report-neuronc-type"/>

        <xsl:call-template name="report-formats"/>
        <xsl:call-template name="report-referrers"/>

        </table>
    </xsl:template>

    <xsl:template name="Generate-Cpt-Core">
        <xsl:comment> template name="Generate-Cpt-Core" </xsl:comment>
        <!-- details table -->
        <table width="100%" border="0" cellspacing="0" cellpadding="3" bgcolor="#F0F7FF">

        <xsl:call-template name="verbose-resource-description"/>
        <xsl:call-template name="report-standard-aspects"/>

        <xsl:apply-templates select="min"/>
        <xsl:apply-templates select="max"/>
        <xsl:apply-templates select="default"/>
        <xsl:apply-templates select="invalid"/>

        <xsl:call-template name="report-neuronc-type"/>

        <xsl:call-template name="report-formats"/>
        <xsl:call-template name="report-referrers"/>

        </table>
    </xsl:template>

    <xsl:template name="Generate-Fpt-Core">
        <xsl:comment> template name="Generate-Fpt-Core" </xsl:comment>
        <!-- details table -->
        <table width="100%" border="0" cellspacing="0" cellpadding="3" bgcolor="#F0F7FF">

        <tr valign="top" align="left">
            <td colspan="4" width="100%">
                <xsl:apply-templates select="name"/>
                <xsl:apply-templates select="unit"/>.<xsl:value-of select="$spacer"/>
                <xsl:apply-templates select="comment"/><xsl:value-of select="$spacer"/>
                <br/>
                <xsl:call-template name="insert-usage-hint">
                    <xsl:with-param name="id" select="@id"/>
                </xsl:call-template>
            </td>       </tr>
        <tr>
            <td colspan="4" width="100%">
                <xsl:value-of select="$spacer"/>
            </td>
        </tr>
        <tr valign="top" align="left">
            <td width="50%" colspan="2">
                <a name="nvs"><h3>Network Variables</h3></a>
            </td>
            <td width="50%" colspan="2">
                <a name="cps"><h3>Configuration Properties</h3></a>
            </td>
        </tr>
        <tr valign="top" align="left">
            <td width="50%" colspan="2">
                <xsl:call-template name="list-fpt-members">
                    <xsl:with-param name="group" select="f-nvs/f-nv[mandatory='yes']"/>
                </xsl:call-template>
                <xsl:call-template name="list-fpt-members">
                    <xsl:with-param name="group" select="f-nvs/f-nv[mandatory='no']"/>
                </xsl:call-template>
            </td>
            <td width="50%" colspan="2">
                <xsl:call-template name="list-fpt-members">
                    <xsl:with-param name="group" select="f-cps/f-cp[mandatory='yes']"/>
                </xsl:call-template>
                <xsl:call-template name="list-fpt-members">
                    <xsl:with-param name="group" select="f-cps/f-cp[mandatory='no']"/>
                </xsl:call-template>
            </td>
        </tr>
        <tr>
            <td colspan="4" width="100%">
                <xsl:value-of select="$spacer"/>
            </td>
        </tr>
        <tr valign="top" align="left">
            <td colspan="4" width="100%">
                <a name="Details"><h2>Details:</h2></a>
            </td>
        </tr>   

        <xsl:call-template name="report-standard-aspects"/>

        <xsl:call-template name="report-standard-aspect">
            <xsl:with-param name="label" select="'Key'"/>
            <xsl:with-param name="value" select="@key"/>
        </xsl:call-template>
        <tr valign="top" align="left">
            <td width="25%">

                    Inheriting:

            </td>
            <td colspan="3" width="75%">
                <i>
                    <xsl:apply-templates select="inheriting/literal"/>
                    <xsl:apply-templates select="inheriting/ref"/>
                </i>
            </td>
        </tr>

        <xsl:apply-templates select="principal"/>

        <xsl:call-template name="report-fpt-members">
            <xsl:with-param name="group" select="f-nvs/f-nv[mandatory='yes']"/>
            <xsl:with-param name="label" select="'Mandatory Network Variables'"/>
        </xsl:call-template>

        <xsl:call-template name="report-fpt-members">
            <xsl:with-param name="group" select="f-nvs/f-nv[mandatory='no']"/>
            <xsl:with-param name="label" select="'Optional Network Variables'"/>
        </xsl:call-template>

        <xsl:call-template name="report-fpt-members">
            <xsl:with-param name="group" select="f-cps/f-cp[mandatory='yes']"/>
            <xsl:with-param name="label" select="'Mandatory Configuration Properties'"/>
        </xsl:call-template>

        <xsl:call-template name="report-fpt-members">
            <xsl:with-param name="group" select="f-cps/f-cp[mandatory='no']"/>
            <xsl:with-param name="label" select="'Optional Configuration Properties'"/>
        </xsl:call-template>

        <xsl:call-template name="report-formats"/>
        <xsl:call-template name="report-referrers"/>

        </table>
    </xsl:template>

    <xsl:template name="Generate-Enumt-Core">
        <xsl:comment> template name="Generate-Enumt-Core" </xsl:comment>
        <!-- details table -->
        <table width="100%" border="0" cellspacing="0" cellpadding="3" bgcolor="#F0F7FF">

        <xsl:call-template name="verbose-resource-description"/>
        <xsl:call-template name="report-standard-aspects"/>

        <xsl:call-template name="report-standard-aspect">
            <xsl:with-param name="label" select="'Headerfile'"/>
            <xsl:with-param name="value" select="h-file"/>
        </xsl:call-template>

        <xsl:variable name="enumt" select="@id"/>

        <xsl:for-each select="members/e-member">
            <tr>
                <td width="25%">
                    <xsl:value-of select="$spacer"/>
                </td>
                <td width="25%">
                    <xsl:value-of select="@id"/>
                </td>
                <td width="25%">
                    <xsl:value-of select="value"/>
                </td>
                <td width="25%">
                    <xsl:apply-templates select="comment"/>
                    <xsl:call-template name="insert-usage-hint">
                        <xsl:with-param name="id">
                            <xsl:value-of select="$enumt"/>::<xsl:value-of select="@id"/>
                        </xsl:with-param>
                    </xsl:call-template>
                </td>
            </tr>
        </xsl:for-each>

        <xsl:call-template name="report-formats"/>
        <xsl:call-template name="report-referrers"/>

        </table>
    </xsl:template>         

    <xsl:template name="verbose-resource-description">
        <xsl:comment> template name="verbose-resource-description" </xsl:comment>
        <tr valign="top" align="left">
            <td colspan="4" width="100%">
                <a name="Overview"><h2>Overview:</h2></a>
                <xsl:apply-templates select="name"/>
                <xsl:apply-templates select="unit"/>.<xsl:value-of select="$spacer"/>
                <xsl:apply-templates select="comment"/><xsl:value-of select="$spacer"/>
                <br/>
                <xsl:call-template name="insert-usage-hint">
                    <xsl:with-param name="id" select="@id"/>
                </xsl:call-template>
                <br/>
                <a name="Details"><h2>Details:</h2></a>
            </td>
        </tr>   
    </xsl:template>

    <xsl:template name="report-standard-aspect">
        <xsl:param name="label"/>
        <xsl:param name="value"/>
        <xsl:comment> template name="report-standard-aspect" </xsl:comment>

        <tr valign="top" align="left">
            <td width="25%">

                    <xsl:value-of select="$label"/>:

            </td>
            <td colspan="3" width="75%">
                <i>
                    <xsl:value-of select="$value"/>
                </i>
            </td>
        </tr>
    </xsl:template>

    <xsl:template name="report-standard-aspects">
        <xsl:comment> template name="report-standard-aspects" </xsl:comment>
        <xsl:if test="@std">
            <!-- note "std" is an attribute, not an element!  -->
            <!-- also note enumeration types don't have such attribute.     -->
            <xsl:call-template name="report-standard-aspect">
                <xsl:with-param name="label" select="'Standard'"/>
                <xsl:with-param name="value" select="@std"/>
            </xsl:call-template>
        </xsl:if>
        <xsl:call-template name="report-standard-aspect">
            <!-- note "idx" is an attribute, not an element!  -->
            <xsl:with-param name="label" select="'Resource Set'"/>
            <xsl:with-param name="value" select="$set"/>
        </xsl:call-template>
        <xsl:call-template name="report-standard-aspect">
            <!-- note "idx" is an attribute, not an element!  -->
            <xsl:with-param name="label" select="'Index'"/>
            <xsl:with-param name="value" select="@idx"/>
        </xsl:call-template>
        <xsl:if test="obsolete">
            <!-- note "obsolete" does not exist for "enumt" -->
            <xsl:call-template name="report-standard-aspect">
                <xsl:with-param name="label" select="'Obsolete'"/>
                <xsl:with-param name="value" select="obsolete"/>
            </xsl:call-template>
        </xsl:if>
        <xsl:if test="size">
            <xsl:call-template name="report-standard-aspect">
                <xsl:with-param name="label" select="'Size'"/>
                <xsl:with-param name="value" select="size"/>
            </xsl:call-template>
        </xsl:if>
        <xsl:if test="p-name">
            <xsl:call-template name="report-standard-aspect">
                <xsl:with-param name="label" select="'Programmatic Name'"/>
                <xsl:with-param name="value" select="p-name"/>
            </xsl:call-template>
        </xsl:if>
    </xsl:template>

    <xsl:template match="name | comment">
        <xsl:comment> template match="name" </xsl:comment>
        <xsl:value-of select="key('strings', @id)"/>
    </xsl:template>

    <xsl:template match="unit">
        <xsl:comment> template match="unit" </xsl:comment>
        <xsl:value-of select="$spacer"/>
        (<xsl:value-of select="key('strings', @id)"/>)
    </xsl:template>

    <xsl:template match="min">
        <xsl:comment> template match="min" </xsl:comment>
        <xsl:call-template name="report-standard-aspect">
            <xsl:with-param name="label" select="'Minimum'"/>
            <xsl:with-param name="value" select="."/>
        </xsl:call-template>
    </xsl:template>

    <xsl:template match="max">
        <xsl:comment> template match="max" </xsl:comment>
        <xsl:call-template name="report-standard-aspect">
            <xsl:with-param name="label" select="'Maximum'"/>
            <xsl:with-param name="value" select="."/>
        </xsl:call-template>
    </xsl:template>

    <xsl:template match="invalid">
        <xsl:comment> template match="invalid" </xsl:comment>
        <xsl:call-template name="report-standard-aspect">
            <xsl:with-param name="label" select="'Invalid'"/>
            <xsl:with-param name="value" select="."/>
        </xsl:call-template>
    </xsl:template>

    <xsl:template match="default">
        <xsl:comment> template match="default" </xsl:comment>
        <xsl:call-template name="report-standard-aspect">
            <xsl:with-param name="label" select="'Default'"/>
            <xsl:with-param name="value" select="."/>
        </xsl:call-template>
    </xsl:template>

    <xsl:template name="report-referrers">
        <xsl:comment> template name="report-referrers" </xsl:comment>
        <xsl:if test="refs">
            <tr valign="top" align="left">
                <td width="25%">

                        Used by:

                </td>
                <td colspan="3" width="75%">
                    <i>
                        <xsl:apply-templates select="refs"/>
                    </i>
                </td>
            </tr>
        </xsl:if>
    </xsl:template>

    <xsl:template name="report-formats">
        <xsl:comment> template name="report-formats" </xsl:comment>
        <xsl:if test="formats">
            <tr valign="top" align="left">
                <td width="25%">

                        Formats:

                </td>
                <td colspan="3" width="75%">
                    <i>
                        <xsl:apply-templates select="formats"/>
                    </i>
                </td>
            </tr>
        </xsl:if>
    </xsl:template>

    <xsl:template match="scaling">
        <xsl:comment> template match="scaling" </xsl:comment>
        <tr valign="top" align="left">
            <td width="25%">

                    Scaling (A,B,C):

            </td>
            <td colspan="3" width="75%">
                <i>
                    <xsl:value-of select="a"/>,<xsl:value-of select="$spacer"/>
                    <xsl:value-of select="b"/>,<xsl:value-of select="$spacer"/>
                    <xsl:value-of select="c"/>
                </i>
            </td>
        </tr><tr valign="top" align="left">
            <td width="25%">

                    Scaled value:

            </td>
            <td colspan="3" width="75%">
                <i>
                    <xsl:value-of select="a"/>
                    *10 <sup><xsl:value-of select="b"/></sup>
                    *(Raw+<xsl:value-of select="c"/>)
                </i>
            </td>
        </tr><tr valign="top" align="left">
            <td width="25%">

                    Resolution:

            </td>
            <td colspan="3" width="75%">
                <i>
                    <xsl:value-of select="s"/>
                </i>
            </td>
        </tr>
    </xsl:template>

    <xsl:template match="scalar">
        <xsl:comment> template match="scalar" </xsl:comment>

        <table valign="top" align="left">
            <tr>
                <td width="100%" colspan="4">
                    <xsl:value-of select="nc-type"/>
                    <xsl:if test="array-size">
                        [<xsl:value-of select="array-size"/>]
                    </xsl:if>
                </td>
            </tr>
            <xsl:apply-templates select="min"/>
            <xsl:apply-templates select="max"/>
            <xsl:apply-templates select="invalid"/>
            <xsl:apply-templates select="scaling"/>
        </table>
    </xsl:template>

    <xsl:template match="float">
        <xsl:comment> template match="float" </xsl:comment>
        <table valign="top" align="left">
            <tr>
                <td width="100%" colspan="4">
                    <xsl:choose>
                        <xsl:when test="nc-type">
                            <xsl:value-of select="nc-type"/>
                        </xsl:when>
                        <xsl:otherwise>
                            float
                        </xsl:otherwise>    
                    </xsl:choose>
                    <xsl:if test="array-size">
                        [<xsl:value-of select="array-size"/>]
                    </xsl:if>
                </td>
            </tr>
            <xsl:apply-templates select="min"/>
            <xsl:apply-templates select="max"/>
        </table>
    </xsl:template>

    <xsl:template match="inheriting">
        <xsl:comment> template match="inheriting" </xsl:comment>
        Inheriting from network variable
    </xsl:template>

    <xsl:template match="ref">
        <xsl:comment> template match="ref" </xsl:comment>
        <table valign="top" align="left">
            <tr>
                <td width="100%" colspan="4">
                    <i>
                    <xsl:choose>
                        <xsl:when test="@href">
                            <!-- a hot link -->

                            <a href="javascript:Transform('{@href}', '{$item-transformation}', '/ldrf/items/*[@id=&quot;{.}&quot;]')">
                                <xsl:value-of select="."/>
                            </a>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="."/>
                        </xsl:otherwise>
                    </xsl:choose>
                    <xsl:if test="array-size">
                        [<xsl:value-of select="array-size"/>]
                    </xsl:if>
                    </i>
                </td>
            </tr>
        </table>
    </xsl:template>

    <xsl:template match="enum">
        <xsl:comment> template match="enum" </xsl:comment>
        <xsl:apply-templates select="ref"/>
        <xsl:if test="array-size">
            [<xsl:value-of select="array-size"/>]
        </xsl:if>
    </xsl:template>

    <xsl:template match="bitfield">
        <xsl:comment> template match="bitfield" </xsl:comment>
        <table valign="top" align="left">
            <tr>
                <td width="100%" colspan="4">
                    bitfield
                </td>
            </tr>
            <xsl:call-template name="report-standard-aspect">
                <xsl:with-param name="label" select="'Signed'"/>
                <xsl:with-param name="value" select="signed"/>
            </xsl:call-template>
            <xsl:call-template name="report-standard-aspect">
                <xsl:with-param name="label" select="'Width'"/>
                <xsl:with-param name="value" select="width"/>
            </xsl:call-template>
            <xsl:call-template name="report-standard-aspect">
                <xsl:with-param name="label" select="'Offset'"/>
                <xsl:with-param name="value" select="offset"/>
            </xsl:call-template>
            <xsl:apply-templates select="min"/>
            <xsl:apply-templates select="max"/>
            <xsl:apply-templates select="scaling"/>
        </table>
    </xsl:template>

    <xsl:template match="aggregate">
        <xsl:comment> template match="aggregate" </xsl:comment>
        <table valign="top" align="left" border="1" bordercolorlight="gray" bordercolordark="gray" cellspacing="0" cellpadding="2">
            <tr>
                <td width="100%" colspan="4">
                    <xsl:choose>
                        <xsl:when test="@cat=&apos;union&apos;">
                            Union
                        </xsl:when>
                        <xsl:otherwise>
                            Structure
                        </xsl:otherwise>
                    </xsl:choose>
                    <xsl:if test="array-size">
                        [<xsl:value-of select="array-size"/>]
                    </xsl:if>
                </td>
            </tr>
            <xsl:for-each select="members/member">
                <tr>
                    <td width="25%">
                        <xsl:value-of select="p-name"/>
                        <xsl:if test="array-size">
                            [<xsl:value-of select="array-size"/>]
                        </xsl:if>
                    </td>
                    <td width="75%" colspan="3" border="0">
                        <xsl:apply-templates select="name"/>
                        <xsl:apply-templates select="unit"/>.<xsl:value-of select="$spacer"/>
                        <xsl:apply-templates select="comment"/><xsl:value-of select="$spacer"/>
                        <br/>
                        <xsl:call-template name="insert-usage-hint">
                            <xsl:with-param name="id">
                                <xsl:value-of select="ancestor::*[@idx]/@id"/>
                                <xsl:call-template name="get-full-member-name"/>
                            </xsl:with-param>
                        </xsl:call-template>
                    </td>
                </tr>
                <tr cellpadding="0" cellspacing="0">
                    <td width="25%">
                        <xsl:value-of select="$spacer"/>
                    </td>
                    <td width="75%" colspan="3" border="0">
                        <xsl:apply-templates select="aggregate"/>
                        <xsl:apply-templates select="scalar"/>
                        <xsl:apply-templates select="float"/>
                        <xsl:apply-templates select="ref"/>
                        <xsl:apply-templates select="bitfield"/>
                        <xsl:apply-templates select="enum"/>
                        <xsl:apply-templates select="inheriting"/>
                    </td>
                </tr>
        </xsl:for-each>
        </table>
    </xsl:template>

    <xsl:template name="report-neuronc-type">
        <xsl:comment> template name="report-neuronc-type" </xsl:comment>
        <tr valign="top" align="left">
            <td width="25%">

                    Neuron C Type:

            </td>
            <td colspan="3" width="75%">
                <xsl:apply-templates select="aggregate"/>
                <xsl:apply-templates select="scalar"/>
                <xsl:apply-templates select="float"/>
                <xsl:apply-templates select="ref"/>
                <xsl:apply-templates select="bitfield"/>
                <xsl:apply-templates select="enum"/>
                <xsl:apply-templates select="inheriting"/>
            </td>
        </tr>
    </xsl:template>

    <xsl:template name="report-fpt-members">
        <xsl:param name="group"/>
        <xsl:param name="label"/>
        <xsl:comment> template name="report-fpt-members" </xsl:comment>

        <xsl:if test="count($group) > 0">
            <tr>
                <td width="100%" colspan="4">
                    <p><br/></p>    
                    <h2>
                        <xsl:value-of select="$label"/>
                    </h2>
                </td>
            </tr>
            <xsl:for-each select="$group">
                <xsl:sort select="@RRG_TAG_IDX" data-type="number"/>
                <tr border="1" bordercolor="gray">
                    <td width="25%" valign="top">
                        <a>
                            <xsl:attribute name="name">
                                <xsl:value-of select="@id"/>
                            </xsl:attribute>
                            <xsl:value-of select="@id"/>
                        </a>
                    </td>
                    <td width="75%" colspan="3">
                        <table width="100%" border="0">
                            <tr>
                                <td width="100%" colspan="4">
                                    <xsl:apply-templates select="name"/><xsl:value-of select="$spacer"/>
                                    <xsl:apply-templates select="comment"/><xsl:value-of select="$spacer"/>
                                    <br/>
                                    <xsl:call-template name="insert-usage-hint">
                                        <xsl:with-param name="id">
                                            <!-- fblock name dot member name -->
                                            <xsl:value-of select="../../@id"/>::<xsl:value-of select="@id"/>
                                        </xsl:with-param>
                                    </xsl:call-template>
                                </td>
                            </tr>
                            <xsl:apply-templates select="member"/>
                            <tr>
                                <td width="25%">
                                    Base type:
                                </td>
                                <td width="75%" colspan="3">
                                    <xsl:apply-templates select="ref"/>
                                </td>
                            </tr>
                            <xsl:apply-templates select="applies"/>
                            <xsl:apply-templates select="restrictions"/>
                            <xsl:apply-templates select="array"/>
                            <xsl:apply-templates select="dir"/>
                            <xsl:apply-templates select="service"/>
                            <xsl:apply-templates select="min"/>
                            <xsl:apply-templates select="max"/>
                            <xsl:apply-templates select="default"/>
                            <xsl:apply-templates select="invalid"/>
                            <tr>
                                <td width="100%" colspan="4" align="right">
                                    <a href="#top">Top of page</a>
                                </td>
                            </tr>
                        </table>
                    </td>
                </tr>
            </xsl:for-each>
        </xsl:if>
    </xsl:template>

    <xsl:template name="list-fpt-members">
        <xsl:param name="group"/>
        <xsl:comment> template name="list-fpt-members" </xsl:comment>

        <xsl:if test="count($group)">
            <xsl:for-each select="$group">
                <xsl:sort select="@id"/>
                <a>

                    <xsl:attribute name="href">#<xsl:value-of select="@id"/>
                    </xsl:attribute>
                    <xsl:value-of select="@id"/>
                </a>
                <br/>
            </xsl:for-each>
            <br/>
        </xsl:if>
    </xsl:template>

    <xsl:template match="member">
        <xsl:comment> match="member" </xsl:comment>
        <xsl:call-template name="report-standard-aspect">
            <xsl:with-param name="label" select="'Member Number'"/>
            <xsl:with-param name="value" select="."/>
        </xsl:call-template>
    </xsl:template>

    <xsl:template match="applies">
        <xsl:comment> match="applies" </xsl:comment>
        <tr valign="top" align="left">
            <td width="25%">

                    Applies to

            </td>
            <td colspan="3" width="75%">
                <i>
                    <a>

                        <xsl:attribute name="href">#<xsl:value-of select="."/></xsl:attribute>
                        <xsl:value-of select="."/>
                    </a>
                </i>
            </td>
        </tr>
    </xsl:template>

    <xsl:template match="principal">
        <xsl:comment> match="principal" </xsl:comment>
        <tr valign="top" align="left">
            <td width="25%">

                    Principal Variable

            </td>
            <td colspan="3" width="75%">
                <i>
                    <a>

                        <xsl:attribute name="href">#<xsl:value-of select="."/></xsl:attribute>
                        <xsl:value-of select="."/>
                    </a>
                </i>
            </td>
        </tr>
    </xsl:template>

    <xsl:template match="restrictions">
        <xsl:comment> match="restrictions" </xsl:comment>
        <xsl:call-template name="report-standard-aspect">
            <xsl:with-param name="label" select="'Flags'"/>
            <xsl:with-param name="value" select="."/>
        </xsl:call-template>
    </xsl:template>

    <xsl:template match="array">
        <xsl:comment> match="array" </xsl:comment>
        <xsl:call-template name="report-standard-aspect">
            <xsl:with-param name="label" select="'Array'"/>
            <xsl:with-param name="value" select="."/>
        </xsl:call-template>
    </xsl:template>

    <xsl:template match="dir">
        <xsl:comment> match="dir" </xsl:comment>
        <xsl:call-template name="report-standard-aspect">
            <xsl:with-param name="label" select="'Direction'"/>
            <xsl:with-param name="value" select="."/>
        </xsl:call-template>
    </xsl:template>

    <xsl:template match="service">
        <xsl:comment> match="service" </xsl:comment>
        <xsl:call-template name="report-standard-aspect">
            <xsl:with-param name="label" select="'Service'"/>
            <xsl:with-param name="value" select="."/>
        </xsl:call-template>
    </xsl:template>

    <xsl:template name="get-full-member-name">
        <xsl:comment> template name="get-full-member-name" </xsl:comment>
        <xsl:call-template name="build-full-member-name">
            <xsl:with-param name="ancestors" select="ancestor-or-self::member/p-name"/>
        </xsl:call-template>
    </xsl:template>

    <xsl:template name="build-full-member-name">
        <xsl:param name="ancestors"/>
        <xsl:comment> template name="build-full-member-name" </xsl:comment>
        <!-- reverse the axis amd dumb out the member names in "language order", top level first: -->
        <xsl:value-of select="concat(string('::'), $ancestors[last()])"/>
        <xsl:if test="$ancestors[1] != $ancestors[last()]">
            <xsl:call-template name="build-full-member-name">
                <xsl:with-param name="ancestors" select="$ancestors[position() != last()]"/>
            </xsl:call-template>
        </xsl:if>
    </xsl:template>

    <!-- the following template matches 'literal' nodes, containing nothing but literal data.   -->
    <!-- literal nodes are used whenever a wrapping element is required but not otherwise given -->
    <xsl:template match="literal">
        <xsl:comment> match="literal" </xsl:comment>
        <xsl:value-of select="."/>
    </xsl:template>

    <!-- +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++    -->
    <!-- The following template is a dummy templates that must be implemented although it is not being          -->
    <!-- used in the context of this transformation. The general utilities in rrg.xsl contain conditional calls -->
    <!-- to the following template, and the XSLT engine must be able to resolve all references to templates in  -->
    <!-- all cases.                                                                                             -->
    <!-- +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++    -->
    <xsl:template name="Generate-Core">
        <xsl:call-template name="wrong-core-generator-error"/>
    </xsl:template>

</xsl:stylesheet>

